# -*- coding: utf-8 -*-
#
# Copyright (C) 2008 Max Stewart
# All rights reserved.
#
# This file is part of the TicketValidator plugin for Trac
#
# TicketValidator is free software: you can redistribute it and/or 
# modify it under the terms of the GNU General Public License as 
# published by the Free Software Foundation, either version 3 of 
# the License, or (at your option) any later version.
#
# TicketValidator is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with TicketValidator.  If not, see 
# <http://www.gnu.org/licenses/>.

import pkg_resources

from trac.admin import IAdminPanelProvider
from trac.core import *
from trac.util.translation import _
from trac.ticket.api import TicketSystem
from trac.web.chrome import add_stylesheet
from trac.web.chrome import ITemplateProvider

class TicketValidatorAdminPanelProvider(Component):
    """Provides an admin page for modifying validator settings."""
     
    implements(IAdminPanelProvider)
     
    def get_admin_panels(self, req):
        """Return a list of available admin panels.
        
        The items returned by this function must be tuples of the form
        `(category, category_label, page, page_label)`.
        """
        if req.perm.has_permission('TICKET_ADMIN'):
            yield ('validator', _('Ticket Validator'), 'settings', _('Settings'))

    def render_admin_panel(self, req, category, page, path_info):
        """Processes a request for the TicketValidator admin page."""
        
        if req.method == 'POST':
            
            if 'apply' not in req.args:
                return self._handle_add_remove(req, category, page, path_info)
            
            self._update_config(req)
        
        rules = []
        options = self.config.options('ticketvalidator')
        
        for name, value in options:
            if name.count('.') == 2:
                req_ida = name.find('.')
                if req_ida != -1:
                        req_ida += 1
                        req_idx = name.rfind('.')
                        if req_idx != -1:
                           type_name = name[0:req_ida-1]
                           rules.append({'type': type_name, 'name': name[req_ida:req_idx], 'fields': value})
            else:
                req_idx = name.find('.required')
                rules.append({'name': name[0:req_idx], 'fields': value})

        
        return self._render(req, rules)

    def _get_rules(self, req):
        """Get the list of rules from the request.
        
        :param:req the request
        :type:req Request
        :return: a list containing validator rules"""
        
        tmp = {}
        to_remove = []
        
        for name in req.args:
            key, pn = self._splitname(name)
        
            if key not in ('type', 'name', 'fields', 'remove'):
                continue
        
            if key == 'remove':
                to_remove.append(pn)
            else:
                if pn not in tmp:
                    tmp[pn] = {}
            
                tmp[pn].update({key: req.args[name]})

        rules = []
        
        for key in tmp:
            if key not in to_remove and tmp[key]['name'] != '':
                rules.append(tmp[key])
                
        return rules

    def _handle_add_remove(self, req, category, page, path_info):
        return self._render(req, self._get_rules(req))

    def _splitname(self, name):
        
        if name.find('_') == -1:
            return name, None
        
        s = name.split('_')
        
        return s[0], s[1]

    def _render(self, req, rules):
        """Render the page."""
        
        rules.append({'': ''})
        
        add_stylesheet(req, 'ticketvalidator/admin.css')
        return 'validator_admin.html', {'rules': rules}
    
    def _update_config(self, req):
        """Save changes to main configuration file."""
        
        for option in self.config['ticketvalidator']:
            self.config.remove('ticketvalidator', option)
        
        rules = self._get_rules(req)
        
        for rule in rules:
	    if rule['type'] == '':
		self.config.set('ticketvalidator', rule['name'] + '.required', rule['fields'])
	    else:
		typename = rule['type'] + '.' + rule['name'] + '.required'
	        self.config.set('ticketvalidator', typename, rule['fields'])
            
        self.config.save()
        

class TicketValidatorChrome(Component):
    """Provides the TicketValidator templates and static resources."""

    implements(ITemplateProvider)

    def get_htdocs_dirs(self):
        """Return the directories containing static resources."""
        return [('ticketvalidator', pkg_resources.resource_filename(__name__, 'htdocs'))]

    def get_templates_dirs(self):
        """Return the directories containing templates."""
        return [pkg_resources.resource_filename(__name__, 'templates')]
